/**
 * List of all Stripe Subscriptions
 *
 * @module package/quiqqer/payment-stripe/bin/controls/backend/settings/Subscriptions
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/payment-stripe/bin/controls/backend/settings/Subscriptions', [

    'qui/controls/Control',
    'qui/controls/loader/Loader',
    'qui/controls/windows/Confirm',
    'qui/controls/buttons/Button',
    'controls/grid/Grid',
    'package/quiqqer/payment-stripe/bin/Stripe',

    'Locale',

    'css!package/quiqqer/payment-stripe/bin/controls/backend/settings/Subscriptions.css'

], function (QUIControl, QUILoader, QUIConfirm, QUIButton, Grid, Stripe, QUILocale) {
    "use strict";

    var lg = 'quiqqer/payment-stripe';

    return new Class({

        Extends: QUIControl,
        Type   : 'package/quiqqer/payment-stripe/bin/controls/backend/settings/Subscriptions',

        Binds: [
            'refresh',
            '$onCreate',
            '$onImport',
            '$onResize',
            '$confirmCancel'
        ],

        initialize: function (options) {
            this.parent(options);

            this.$Grid    = null;
            this.$Content = null;
            this.Loader   = new QUILoader();

            this.$search      = false;
            this.$SearchInput = null;

            this.addEvents({
                onCreate: this.$onCreate,
                onResize: this.$onResize,
                onImport: this.$onImport
            });
        },

        /**
         * Refresh the grid
         */
        refresh: function () {
            if (!this.$Grid) {
                return;
            }

            this.Loader.show();

            var search = this.$SearchInput.value.trim();

            if (search === '') {
                search = false;
            }

            switch (this.$Grid.options.sortOn) {
                case 'active_status':
                    this.$Grid.options.sortOn = 'active';
                    break;

                case 'customer_text':
                    this.$Grid.options.sortOn = 'customer';
                    break;
            }

            return Stripe.getSubscriptions({
                perPage: this.$Grid.options.perPage,
                page   : this.$Grid.options.page,
                sortBy : this.$Grid.options.sortBy,
                sortOn : this.$Grid.options.sortOn,
                search : search
            }).then(function (result) {
                var TableButtons = this.$Grid.getAttribute('buttons');
                TableButtons.cancel.disable();

                var detailsUrl,
                    detailsTitle = QUILocale.get(lg, 'controls.backend.settings.Subscriptions.details_title');

                if (result.isSandbox) {
                    detailsUrl = 'https://dashboard.stripe.com/test/subscriptions/';
                } else {
                    detailsUrl = 'https://dashboard.stripe.com/subscriptions/'
                }

                var $onDetailsClick = function (event) {
                    var Link = new Element('a', {
                        href  : detailsUrl + event.target.get('data-id'),
                        target: '_blank'
                    }).inject(document.body);

                    Link.click();
                    Link.destroy();
                };

                for (var i = 0, len = result.data.length; i < len; i++) {
                    var Row      = result.data[i];
                    var Customer = JSON.decode(Row.customer);

                    Row.customer_text = Customer.firstname + " " + Customer.lastname + " (" + Customer.id + ")";

                    Row.active_status = new Element('span', {
                        'class': parseInt(Row.active) ? 'fa fa-check' : 'fa fa-close'
                    });

                    Row.details = new Element('span', {
                        'class'  : 'fa fa-cc-stripe quiqqer-payment-stripe-subscriptions',
                        title    : detailsTitle,
                        'data-id': Row.stripe_id,
                        events   : {
                            click: $onDetailsClick
                        }
                    });
                }

                this.$Grid.setData(result);
                this.Loader.hide();
            }.bind(this), function () {
                this.destroy();
            }.bind(this));
        },

        /**
         * Event Handling
         */

        $onImport: function () {
            this.$Content = new Element('div', {
                'class': 'quiqqer-payment-stripe-subscriptions field-container-field'
            }).inject(this.getElm(), 'after');

            this.Loader.inject(this.$Content);

            this.$Content.getParent('form').setStyle('height', '100%');
            this.$Content.getParent('table').setStyle('height', '100%');
            this.$Content.getParent('tbody').setStyle('height', '100%');
            this.$Content.getParent('.field-container').setStyle('height', '100%');

            this.create();
            this.$onCreate();
            this.refresh();
        },

        /**
         * event : on panel create
         */
        $onCreate: function () {
            var self = this;

            // Search
            this.$SearchInput = new Element('input', {
                'class'    : 'quiqqer-payment-stripe-subscriptions-search',
                placeholder: QUILocale.get(lg, 'controls.backend.settings.Subscriptions.search.placeholder'),
                events     : {
                    keydown: function (event) {
                        if (typeof event !== 'undefined' &&
                            event.code === 13) {
                            self.refresh();
                        }
                    }
                }
            }).inject(this.$Content);

            // Grid
            var Container = new Element('div', {
                style: {
                    height: '100%',
                    width : '100%'
                }
            }).inject(this.$Content);

            this.$Grid = new Grid(Container, {
                pagination       : true,
                multipleSelection: true,
                serverSort       : true,
                sortOn           : 'stripe_id',
                sortBy           : 'DESC',

                accordion           : false,
                autoSectionToggle   : false,
                openAccordionOnClick: false,

                buttons    : [{
                    name     : 'cancel',
                    text     : QUILocale.get(lg, 'controls.backend.settings.Subscriptions.tbl.btn.cancel'),
                    textimage: 'fa fa-ban',
                    events   : {
                        onClick: this.$confirmCancel
                    }
                }],
                columnModel: [{
                    header   : QUILocale.get(lg, 'controls.backend.settings.Subscriptions.tbl.active_status'),
                    dataIndex: 'active_status',
                    dataType : 'node',
                    width    : 45
                }, {
                    header   : QUILocale.get(lg, 'controls.backend.settings.Subscriptions.tbl.details'),
                    dataIndex: 'details',
                    dataType : 'node',
                    width    : 50
                }, {
                    header   : QUILocale.get(lg, 'controls.backend.settings.Subscriptions.tbl.stripe_id'),
                    dataIndex: 'stripe_id',
                    dataType : 'string',
                    width    : 200
                }, {
                    header   : QUILocale.get(lg, 'controls.backend.settings.Subscriptions.tbl.stripe_billing_plan_id'),
                    dataIndex: 'stripe_billing_plan_id',
                    dataType : 'string',
                    width    : 200
                }, {
                    header   : QUILocale.get(lg, 'controls.backend.settings.Subscriptions.tbl.customer_text'),
                    dataIndex: 'customer_text',
                    dataType : 'string',
                    width    : 200
                }, {
                    header   : QUILocale.get(lg, 'controls.backend.settings.Subscriptions.tbl.global_process_id'),
                    dataIndex: 'global_process_id',
                    dataType : 'string',
                    width    : 250
                }, {
                    dataIndex: 'active',
                    dataType : 'bool',
                    hidden   : true
                }]
            });

            this.$Grid.addEvents({
                onRefresh: this.refresh,

                onClick: function () {
                    var TableButtons = self.$Grid.getAttribute('buttons'),
                        selected     = self.$Grid.getSelectedData().length;

                    if (!Object.getLength(TableButtons)) {
                        return;
                    }

                    if (selected === 1 && parseInt(self.$Grid.getSelectedData()[0].active)) {
                        TableButtons.cancel.enable();
                    } else {
                        TableButtons.cancel.disable();
                    }
                }
            });

            this.$onResize();
        },

        /**
         * event : on panel resize
         */
        $onResize: function () {
            if (!this.$Grid) {
                return;
            }

            var size = this.$Content.getSize();

            this.$Grid.setHeight(size.y - 20);
            this.$Grid.setWidth(size.x - 20);
            this.$Grid.resize();
        },

        /**
         * Confirm Subscription cancellation
         */
        $confirmCancel: function () {
            var self           = this;
            var subscriptionId = this.$Grid.getSelectedData()[0].stripe_id;

            new QUIConfirm({
                maxHeight: 400,
                maxWidth : 600,
                autoclose: false,

                information: QUILocale.get(lg, 'controls.backend.settings.Subscriptions.cancel.information'),
                title      : QUILocale.get(lg, 'controls.backend.settings.Subscriptions.cancel.title', {
                    subscriptionId: subscriptionId
                }),
                texticon   : 'fa fa-ban',
                text       : QUILocale.get(lg, 'controls.backend.settings.Subscriptions.cancel.text', {
                    subscriptionId: subscriptionId
                }),
                icon       : 'fa fa-ban',

                cancel_button: {
                    text     : false,
                    textimage: 'icon-remove fa fa-remove'
                },
                ok_button    : {
                    text     : QUILocale.get(lg, 'controls.backend.settings.Subscriptions.cancel.confirm'),
                    textimage: 'fa fa-ban'
                },

                events: {
                    onOpen  : function (Win) {
                        Win.getButton('submit').getElm().addClass('btn-red');
                    },
                    onSubmit: function (Popup) {
                        Popup.Loader.show();

                        Stripe.cancelSubscription(subscriptionId).then(function (success) {
                            if (!success) {
                                Popup.Loader.hide();
                                return;
                            }

                            Popup.close();
                            self.refresh();
                        }, function () {
                            Popup.Loader.hide();
                        })
                    }
                }
            }).open();
        }
    });
});
