<?php

namespace QUI\ERP\Payments\Stripe;

use QUI;
use QUI\ERP\Accounting\Invoice\Invoice;
use QUI\ERP\Order\AbstractOrder;
use QUI\Users\User;
use Stripe\Customer as StripeCustomer;
use Stripe\PaymentMethod as StripePaymentMethod;
use Stripe\Exception\ApiErrorException;
use QUI\Interfaces\Users\User as UserInterface;

/**
 * Class Utils
 *
 * Utility class for quiqqer/payment-stripe
 */
class Utils
{
    /**
     * Get order amount in smallest currency unit
     *
     * @param AbstractOrder|Invoice $Source
     * @return int
     *
     * @throws \QUI\Exception
     */
    public static function getCentAmount($Source)
    {
        if ($Source instanceof AbstractOrder) {
            $PriceCalculation = $Source->getPriceCalculation();
            $amount           = $PriceCalculation->getSum()->precision(2)->get();
            return $amount * 100; // convert to smallest currency unit
        }

        if ($Source instanceof Invoice) {
            $invoiceAmount = (float)$Source->getAttribute('toPay');
            $AmountValue   = new QUI\ERP\Accounting\CalculationValue($invoiceAmount, $Source->getCurrency(), 2);
            return $AmountValue->get() * 100;
        }

        throw new StripeException('Invalid $Source provided.');
    }

    /**
     * Attaches a Stripe PaymentMethod to a Stripe Customer.
     *
     * If the Customer does not have any previous PaymentMethods, the PaymentMethod is automatically
     * set as the default.
     *
     * @param UserInterface $QuiqqerUser
     * @param string $paymentMethodId
     * @return void
     *
     * @throws StripeException
     * @throws ApiErrorException
     * @throws \QUI\Exception
     */
    public static function attachPaymentMethodToStripeCustomer(UserInterface $QuiqqerUser, string $paymentMethodId)
    {
        // If ERPUser -> get QUIQQER User
        if ($QuiqqerUser instanceof QUI\ERP\User) {
            $QuiqqerUser = QUI::getUsers()->get($QuiqqerUser->getId());
        }

        $stripeCustomerId = $QuiqqerUser->getAttribute(AbstractBasePayment::USER_ATTR_STRIPE_CUSTOMER_ID);

        if (empty($stripeCustomerId)) {
            throw new StripeException([
                'quiqqer/payment-stripe',
                'exception.Utils.attachPaymentMethodToStripeCustomer.no_stripe_customer_id',
                [
                    'userId' => $QuiqqerUser->getId()
                ]
            ]);
        }

        Provider::setupApi();
        $StripePaymentMethod = StripePaymentMethod::retrieve($paymentMethodId);

        try {
            $StripePaymentMethod->attach([
                'customer' => $stripeCustomerId
            ]);
        } catch (ApiErrorException $Exception) {
            if (\mb_strpos($Exception->getMessage(), 'has already been attached')) {
                return;
            }

            throw $Exception;
        }
    }

    /**
     * Get a payment description for order (shown at Stripe dashboard)
     *
     * @param AbstractOrder $Order
     * @return string
     */
    public static function getPaymentDescriptionForOrder(AbstractOrder $Order)
    {
        $defaultDescription = $Order->getCustomer()->getLocale()->get(
            'quiqqer/payment-stripe',
            'PaymentIntent.description.default',
            [
                'orderId' => $Order->getPrefixedId()
            ]
        );

        try {
            $template = Provider::getGeneralSetting('payment_description');

            if (empty($template)) {
                return $defaultDescription;
            }

            $placeholders = ['orderId'];

            foreach ($placeholders as $placeholder) {
                switch ($placeholder) {
                    case 'orderId':
                        $replace = $Order->getPrefixedId();
                        break;

                    default:
                        continue 2;
                }

                $template = \str_replace('{'.$placeholder.'}', $replace, $template);
            }

            return $template;
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            return $defaultDescription;
        }
    }
}
